// Библиотеки отвечающие за режим сна
#include <avr/sleep.h>
#include <avr/power.h>
// Библиотеки для работы с дисплеем 5110
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
// Библиотека работы с устройствами по I2C
#include <Wire.h> 
 

// pin 7 - Serial clock out (SCLK)
// pin 6 - Serial data out (DIN)
// pin 5 - Data/Command select (D/C)
// pin 4 - LCD chip select (CS)
// pin 3 - LCD reset (RST)
Adafruit_PCD8544 display = Adafruit_PCD8544(3, 4, 5, 6, 7);

boolean DebugFlag = true;
int BH1750address = 0x23; //Адрес IIC BH1750
byte buff[2];


int key_count          = 0; 
unsigned long key_time = 0;
boolean key_press      = false;
boolean power_on       = false;


volatile boolean int_pwr_up  = false;
volatile boolean int_key     = false;

/**
* Функция определения напряжения питания устройства
*/
static int vccRead(byte us =250) {
  ADMUX = 1<<REFS0; // опорное напряжение - Vcc
  ADMUX |= 0x0E;    // объект измерения - внутренний источник
                    // стабилизированного напряжения 1.1В
  delayMicroseconds(us);
 
  ADCSRA |= 1<<ADSC;         // запуск АЦ-преобразования
  while(ADCSRA & (1<<ADSC)); // и ожидание его завершения
  word x = ADC;
  return x ? (1100L * 1023) / x : -1;
}


/**
* Функция инициализации при включении питания 
*/
void PowerOn(){
  digitalWrite(8,HIGH);   //  on
  digitalWrite(9,HIGH);   // on
  digitalWrite(13,HIGH);  // power LED on
  display.begin();
  Wire.begin();
  BH1750_Init();
  display.setContrast(37);
}

/**
* Функция выключение всех устройств при переходе в POWER_DOWN моде
*/
void PowerOff(){
// Отключаем все выходы 3-19
  for( int i=3; i<20; i++ ){
     pinMode(i,INPUT_PULLUP); 
     digitalWrite(i,LOW);  
  }
}

/**
* Функция обраюотчик прерывания на кнопку питания 
*/
void intKey()
{
  int_key = true;
  detachInterrupt(0); //Отключаем прерывания 
}

/**
* Функия перевода ARDUINO в POWER_DOWN режим
*/
void EnterSleep()
{
  PowerOff();
  DebugPrint("I am sleeping");
  attachInterrupt(0, intKey, LOW); //Если на 0-вом прерываниии - ноль, то просыпаемся.
  power_on = false;
  delay(100);
  set_sleep_mode(SLEEP_MODE_PWR_DOWN); //Определяем режим сна
  
  sleep_enable(); //Разрешаем спящий режим
  sleep_mode(); //Спим (Прерывания продолжают работать.) Программа останавливается.
  sleep_disable(); //Запрещаем спящий режим
}

/* 
Чтение значение освещенности
*/
int BH1750_Read() //
{
  int i=0;
  Wire.beginTransmission(BH1750address);
  Wire.requestFrom(BH1750address, 2);
  while(Wire.available()) //
  {
    buff[i] = Wire.read();  // Считываем 1байт данных
    i++;
  }
  Wire.endTransmission(); 
  return i;
}
 
/** 
* Инициализация датчика освещенности
*/
void BH1750_Init()
{
  Wire.beginTransmission(BH1750address);
  Wire.write(0x10);//1lx reolution 120ms
  Wire.endTransmission();
}

/** 
* Инициализация датчика освещенности
*/
void BH1750_Init40()
{
  Wire.beginTransmission(BH1750address);
  Wire.write(0x13);//3lx reolution 40ms
  Wire.endTransmission();
}


void DebugPrint( char *msg ){
   if( DebugFlag )Serial.println(msg);
} 


 
void setup()
{
   if( DebugFlag ){
      Serial.begin(9600);
      Serial.println("Hello World");
   }
   pinMode(13,OUTPUT);      // Светодиод питания
   pinMode(9,INPUT_PULLUP); // Выход на подсветку дисплея
   pinMode(8,INPUT_PULLUP); // Выход на питание дисплея

   analogRead(0);           // Порт для измерения напряжения питания по внутреннему источнику
   pinMode(2, INPUT_PULLUP);// Порт кнопки питания 
 //  digitalWrite(pin, HIGH);   
   attachInterrupt(0, intKey, LOW); //Если на 0-вом прерываниии - ноль, то просыпаемся.
   EnterSleep();            // Идем спать
 
}
 
unsigned long ms1 = 0; 
 
void loop()
{
   unsigned long ms = millis();
// Кнопка нажата
   if( key_press ){
// Кнопка наконец то отаущена     
      if( digitalRead(2) == HIGH ){
         unsigned int key_interval =0;
         if( DebugFlag ){
            Serial.print("Press key ");
            key_interval = abs(ms - key_time);
            Serial.print(key_count);
            Serial.print(" ");
            Serial.print(key_interval);
            Serial.println("");
         }
         key_count = 0;
         key_press = false;
         if( power_on && key_interval > 2000 ){
            DebugPrint("Power off");
            EnterSleep();
         }  
         else if( !power_on && key_interval < 2000 ){
            DebugPrint("Sleep again");
            EnterSleep();
         }
         else if( key_interval>2000 ){
           DebugPrint("Power on");
           power_on = true;
           PowerOn();
         }  
         else {
           DebugPrint("Press PWR key");
         }
      }  
   }
// Прерывание нажатия кнопки
   if( int_key ){
      delay(100);
      key_press = true;
// Первый тик времени нажатия кнопки
      if( key_count == 0 )key_time = ms;
      key_count++;
// Заряжаем обработку прерывния по новой
      int_key = false;
      attachInterrupt(0, intKey, LOW); //Если на 0-вом прерываниии - ноль, то просыпаемся.
   }
   
// Опрос датчиков каждую секунду
   if( (ms < ms1 || (ms - ms1) > 1000)&&power_on ){
      ms1 = ms;
      uint16_t l1=0,l2=0,u1=0,l_min=0,l_max=0;
      
      
      BH1750_Init();
      if(2==BH1750_Read()){
          l1=((buff[0]<<8)|buff[1])/1.2;
      }
/*      
      BH1750_Init40();
      for( int i=0; i<100; i++ ){
         if(2==BH1750_Read()){
            l2=((buff[0]<<8)|buff[1])/1.2;
         }
         if( i == 0 ){l_min=l2; l_max=l2;}
         else {
            if( l2 > l_max )l_max = l2;
            if( l2 < l_min )l_min = l2;
         }
      }
*/      
      u1 = vccRead(100);
      if( DebugFlag ){
         Serial.print("Light (Lx)=");
         Serial.print(l1);
         Serial.print("Light2 (Lx)=");
         Serial.print(l2);
         Serial.print("LightMin (Lx)=");
         Serial.print(l_min);
         Serial.print("LightMax (Lx)=");
         Serial.print(l_max);
         Serial.print(",Vcc (mV)=");
         Serial.println(u1);
      }
      display.clearDisplay();
      display.setTextSize(2);
      display.setCursor(0,0);
      display.print(l1);
      display.setTextSize(1);
      display.println(" Lx");
      display.println("");
      display.println("");
      display.println("");
      display.print("Bat: ");
      display.print(u1/1000);
      display.print(".");
      display.print((u1/100)%10);
      display.println(" V");
      display.display(); 
 
   }
 
}
